@@ -36,18 +36,6 @@ module Codec.CBOR.Decoding
   , decodeInt16         -- :: Decoder s Int16
   , decodeInt32         -- :: Decoder s Int32
   , decodeInt64         -- :: Decoder s Int64
-  , decodeWordCanonical      -- :: Decoder s Word
-  , decodeWord8Canonical     -- :: Decoder s Word8
-  , decodeWord16Canonical    -- :: Decoder s Word16
-  , decodeWord32Canonical    -- :: Decoder s Word32
-  , decodeWord64Canonical    -- :: Decoder s Word64
-  , decodeNegWordCanonical   -- :: Decoder s Word
-  , decodeNegWord64Canonical -- :: Decoder s Word64
-  , decodeIntCanonical       -- :: Decoder s Int
-  , decodeInt8Canonical      -- :: Decoder s Int8
-  , decodeInt16Canonical     -- :: Decoder s Int16
-  , decodeInt32Canonical     -- :: Decoder s Int32
-  , decodeInt64Canonical     -- :: Decoder s Int64
   , decodeInteger       -- :: Decoder s Integer
   , decodeFloat         -- :: Decoder s Float
   , decodeDouble        -- :: Decoder s Double
@@ -58,29 +46,18 @@ module Codec.CBOR.Decoding
   , decodeStringIndef   -- :: Decoder s ()
   , decodeUtf8ByteArray -- :: Decoder s ByteArray
   , decodeListLen       -- :: Decoder s Int
-  , decodeListLenCanonical -- :: Decoder s Int
   , decodeListLenIndef  -- :: Decoder s ()
   , decodeMapLen        -- :: Decoder s Int
-  , decodeMapLenCanonical -- :: Decoder s Int
   , decodeMapLenIndef   -- :: Decoder s ()
   , decodeTag           -- :: Decoder s Word
   , decodeTag64         -- :: Decoder s Word64
-  , decodeTagCanonical   -- :: Decoder s Word
-  , decodeTag64Canonical -- :: Decoder s Word64
   , decodeBool          -- :: Decoder s Bool
   , decodeNull          -- :: Decoder s ()
   , decodeSimple        -- :: Decoder s Word8
-  , decodeIntegerCanonical -- :: Decoder s Integer
-  , decodeFloat16Canonical -- :: Decoder s Float
-  , decodeFloatCanonical   -- :: Decoder s Float
-  , decodeDoubleCanonical  -- :: Decoder s Double
-  , decodeSimpleCanonical  -- :: Decoder s Word8
 
   -- ** Specialised Read input token operations
   , decodeWordOf        -- :: Word -> Decoder s ()
   , decodeListLenOf     -- :: Int  -> Decoder s ()
-  , decodeWordCanonicalOf    -- :: Word -> Decoder s ()
-  , decodeListLenCanonicalOf -- :: Int  -> Decoder s ()
 
   -- ** Branching operations
 --, decodeBytesOrIndef
@@ -94,9 +71,41 @@ module Codec.CBOR.Decoding
   , peekAvailable        -- :: Decoder s Int
   , TokenType(..)
 
+  -- ** Canonical CBOR
+  -- $canonical
+  , decodeWordCanonical      -- :: Decoder s Word
+  , decodeWord8Canonical     -- :: Decoder s Word8
+  , decodeWord16Canonical    -- :: Decoder s Word16
+  , decodeWord32Canonical    -- :: Decoder s Word32
+  , decodeWord64Canonical    -- :: Decoder s Word64
+  , decodeNegWordCanonical   -- :: Decoder s Word
+  , decodeNegWord64Canonical -- :: Decoder s Word64
+  , decodeIntCanonical       -- :: Decoder s Int
+  , decodeInt8Canonical      -- :: Decoder s Int8
+  , decodeInt16Canonical     -- :: Decoder s Int16
+  , decodeInt32Canonical     -- :: Decoder s Int32
+  , decodeInt64Canonical     -- :: Decoder s Int64
+  , decodeBytesCanonical -- :: Decoder s ByteString
+  , decodeByteArrayCanonical -- :: Decoder s ByteArray
+  , decodeStringCanonical -- :: Decoder s Text
+  , decodeUtf8ByteArrayCanonical -- :: Decoder s ByteArray
+  , decodeListLenCanonical -- :: Decoder s Int
+  , decodeMapLenCanonical -- :: Decoder s Int
+  , decodeTagCanonical   -- :: Decoder s Word
+  , decodeTag64Canonical -- :: Decoder s Word64
+  , decodeIntegerCanonical -- :: Decoder s Integer
+  , decodeFloat16Canonical -- :: Decoder s Float
+  , decodeFloatCanonical   -- :: Decoder s Float
+  , decodeDoubleCanonical  -- :: Decoder s Double
+  , decodeSimpleCanonical  -- :: Decoder s Word8
+  , decodeWordCanonicalOf    -- :: Word -> Decoder s ()
+  , decodeListLenCanonicalOf -- :: Int  -> Decoder s ()
+
+{-
   -- ** Special operations
---, ignoreTerms
---, decodeTrace
+  , ignoreTerms
+  , decodeTrace
+-}
 
   -- * Sequence operations
   , decodeSequenceLenIndef -- :: ...
@@ -188,11 +197,15 @@ data DecodeAction s a
     | ConsumeBool          (Bool      -> ST s (DecodeAction s a))
     | ConsumeSimple        (Word#     -> ST s (DecodeAction s a))
 
-    | ConsumeIntegerCanonical (Integer -> ST s (DecodeAction s a))
-    | ConsumeFloat16Canonical (Float#  -> ST s (DecodeAction s a))
-    | ConsumeFloatCanonical   (Float#  -> ST s (DecodeAction s a))
-    | ConsumeDoubleCanonical  (Double# -> ST s (DecodeAction s a))
-    | ConsumeSimpleCanonical  (Word#   -> ST s (DecodeAction s a))
+    | ConsumeIntegerCanonical       (Integer -> ST s (DecodeAction s a))
+    | ConsumeFloat16Canonical       (Float#  -> ST s (DecodeAction s a))
+    | ConsumeFloatCanonical         (Float#  -> ST s (DecodeAction s a))
+    | ConsumeDoubleCanonical        (Double# -> ST s (DecodeAction s a))
+    | ConsumeBytesCanonical         (ByteString-> ST s (DecodeAction s a))
+    | ConsumeByteArrayCanonical     (ByteArray -> ST s (DecodeAction s a))
+    | ConsumeStringCanonical        (Text      -> ST s (DecodeAction s a))
+    | ConsumeUtf8ByteArrayCanonical (ByteArray -> ST s (DecodeAction s a))
+    | ConsumeSimpleCanonical        (Word#   -> ST s (DecodeAction s a))
 
     | ConsumeBytesIndef   (ST s (DecodeAction s a))
     | ConsumeStringIndef  (ST s (DecodeAction s a))
@@ -289,6 +302,38 @@ getDecodeAction :: Decoder s a -> ST s (DecodeAction s a)
 getDecodeAction (Decoder k) = k (\x -> return (Done x))
 
 
+-- $canonical
+--
+-- <https://tools.ietf.org/html/rfc7049#section-3.9>
+--
+-- In general in CBOR there can be multiple representations for the same value,
+-- for example the integer @0@ represented in 8, 16, 32 or 64 bits. This
+-- library always encodeds values in the shortest representation but on
+-- decoding allows any valid encoding. For some applications it is useful or
+-- important to only decode the canonical encoding. The decoder primitves here
+-- are to allow applications to implement canonical decoding.
+--
+-- It is important to note that achieving a canonical representation is /not/
+-- simply about using these primitives. For example consider a typical CBOR
+-- encoding of a Haskell @Set@ data type. This will be encoded as a CBOR list
+-- of the set elements. A typical implementation might be:
+--
+-- > encodeSet = encodeList . Set.toList
+-- > decodeSet = fmap Set.fromList . decodeList
+--
+-- This /does not/ enforce a canonical encoding. The decoder above will allow
+-- set elements in any order. The use of @Set.fromList@ forgets the order.
+-- To enforce that the decoder only accepts the canonical encoding it will
+-- have to check that the elemets in the list are /strictly/ increasing.
+-- Similar issues arise in many other data types, wherever there is redundancy
+-- in the external representation.
+--
+-- The decoder primitives in this section are not much more expensive than
+-- their normal counterparts. If checking the canonical encoding property is
+-- critical then a technique that is more expensive but easier to implement and
+-- test is to decode normally, re-encode and check the serialised bytes are the
+-- same.
+
 ---------------------------------------
 -- Read input tokens of various types
 --
@@ -519,6 +564,13 @@ decodeBytes :: Decoder s ByteString
 decodeBytes = Decoder (\k -> return (ConsumeBytes (\bs -> k bs)))
 {-# INLINE decodeBytes #-}
 
+-- | Decode canonical representation of a string of bytes as a @'ByteString'@.
+--
+-- @since 0.2.1.0
+decodeBytesCanonical :: Decoder s ByteString
+decodeBytesCanonical = Decoder (\k -> return (ConsumeBytesCanonical (\bs -> k bs)))
+{-# INLINE decodeBytesCanonical #-}
+
 -- | Decode a token marking the beginning of an indefinite length
 -- set of bytes.
 --
@@ -538,6 +590,17 @@ decodeByteArray :: Decoder s ByteArray
 decodeByteArray = Decoder (\k -> return (ConsumeByteArray k))
 {-# INLINE decodeByteArray #-}
 
+-- | Decode canonical representation of a string of bytes as a 'ByteArray'.
+--
+-- Also note that this will eagerly copy the content out of the input
+-- to ensure that the input does not leak in the event that the 'ByteArray' is
+-- live but not forced.
+--
+-- @since 0.2.1.0
+decodeByteArrayCanonical :: Decoder s ByteArray
+decodeByteArrayCanonical = Decoder (\k -> return (ConsumeByteArrayCanonical k))
+{-# INLINE decodeByteArrayCanonical #-}
+
 -- | Decode a textual string as a piece of @'Text'@.
 --
 -- @since 0.2.0.0
@@ -545,6 +608,13 @@ decodeString :: Decoder s Text
 decodeString = Decoder (\k -> return (ConsumeString (\str -> k str)))
 {-# INLINE decodeString #-}
 
+-- | Decode canonical representation of a textual string as a piece of @'Text'@.
+--
+-- @since 0.2.1.0
+decodeStringCanonical :: Decoder s Text
+decodeStringCanonical = Decoder (\k -> return (ConsumeStringCanonical (\str -> k str)))
+{-# INLINE decodeStringCanonical #-}
+
 -- | Decode a token marking the beginning of an indefinite length
 -- string.
 --
@@ -565,6 +635,18 @@ decodeUtf8ByteArray :: Decoder s ByteArray
 decodeUtf8ByteArray = Decoder (\k -> return (ConsumeUtf8ByteArray k))
 {-# INLINE decodeUtf8ByteArray #-}
 
+-- | Decode canonical representation of a textual string as UTF-8 encoded
+-- 'ByteArray'. Note that the result is not validated to be well-formed UTF-8.
+--
+-- Also note that this will eagerly copy the content out of the input
+-- to ensure that the input does not leak in the event that the 'ByteArray' is
+-- live but not forced.
+--
+-- @since 0.2.1.0
+decodeUtf8ByteArrayCanonical :: Decoder s ByteArray
+decodeUtf8ByteArrayCanonical = Decoder (\k -> return (ConsumeUtf8ByteArrayCanonical k))
+{-# INLINE decodeUtf8ByteArrayCanonical #-}
+
 -- | Decode the length of a list.
 --
 -- @since 0.2.0.0
