@@ -301,10 +301,15 @@ fromFlatTerm decoder ft =
     go (TkBool    b : ts) (ConsumeBool   k)        = k b >>= go ts
     go (TkSimple  n : ts) (ConsumeSimple k)        = k (unW8# n) >>= go ts
 
-    go (TkFloat16 f : ts) (ConsumeFloat16Canonical k) = k (unF# f) >>= go ts
-    go (TkFloat32 f : ts) (ConsumeFloatCanonical   k) = k (unF# f) >>= go ts
-    go (TkFloat64 f : ts) (ConsumeDoubleCanonical  k) = k (unD# f) >>= go ts
-    go (TkSimple  n : ts) (ConsumeSimpleCanonical  k) = k (unW8# n) >>= go ts
+    go (TkFloat16 f : ts) (ConsumeFloat16Canonical k)       = k (unF# f) >>= go ts
+    go (TkFloat32 f : ts) (ConsumeFloatCanonical   k)       = k (unF# f) >>= go ts
+    go (TkFloat64 f : ts) (ConsumeDoubleCanonical  k)       = k (unD# f) >>= go ts
+    go (TkBytes  bs : ts) (ConsumeBytesCanonical  k)        = k bs >>= go ts
+    go (TkBytes  bs : ts) (ConsumeByteArrayCanonical k)     = k (BA.fromByteString bs) >>= go ts
+    go (TkString st : ts) (ConsumeStringCanonical k)        = k st >>= go ts
+    go (TkString st : ts) (ConsumeUtf8ByteArrayCanonical k) = k (BA.fromByteString $ TE.encodeUtf8 st)
+                                                              >>= go ts
+    go (TkSimple  n : ts) (ConsumeSimpleCanonical  k)       = k (unW8# n) >>= go ts
 
     go (TkBytesBegin  : ts) (ConsumeBytesIndef   da) = da >>= go ts
     go (TkStringBegin : ts) (ConsumeStringIndef  da) = da >>= go ts
@@ -371,10 +376,14 @@ fromFlatTerm decoder ft =
     go ts (ConsumeBool   _) = unexpected "decodeBool"   ts
     go ts (ConsumeSimple _) = unexpected "decodeSimple" ts
 
-    go ts (ConsumeFloat16Canonical _) = unexpected "decodeFloat16Canonical" ts
-    go ts (ConsumeFloatCanonical   _) = unexpected "decodeFloatCanonical"   ts
-    go ts (ConsumeDoubleCanonical  _) = unexpected "decodeDoubleCanonical"  ts
-    go ts (ConsumeSimpleCanonical  _) = unexpected "decodeSimpleCanonical"  ts
+    go ts (ConsumeFloat16Canonical _)       = unexpected "decodeFloat16Canonical"       ts
+    go ts (ConsumeFloatCanonical   _)       = unexpected "decodeFloatCanonical"         ts
+    go ts (ConsumeDoubleCanonical  _)       = unexpected "decodeDoubleCanonical"        ts
+    go ts (ConsumeBytesCanonical  _)        = unexpected "decodeBytesCanonical"         ts
+    go ts (ConsumeByteArrayCanonical _)     = unexpected "decodeByteArrayCanonical"     ts
+    go ts (ConsumeStringCanonical _)        = unexpected "decodeStringCanonical"        ts
+    go ts (ConsumeUtf8ByteArrayCanonical _) = unexpected "decodeUtf8ByteArrayCanonical" ts
+    go ts (ConsumeSimpleCanonical  _)       = unexpected "decodeSimpleCanonical"        ts
 
 #if defined(ARCH_32bit)
     -- 64bit variants for 32bit machines
